==> excel2xoonips-1.0.0.tar.gz: excel2xoonips-1.0.0/excel2xoonips/bsd_mktemp.c <==
/*
 * Copyright (c) 1987, 1993
 *    The Regents of the University of California.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 	This product includes software developed by the University of
 * 	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Portions Copyright (c) 1993 by Digital Equipment Corporation.
 * 
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies, and that
 * the name of Digital Equipment Corporation not be used in advertising or
 * publicity pertaining to distribution of the document or software without
 * specific, written prior permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <sys/types.h>
#include <sys/stat.h>

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif

#ifdef WIN32
# define mkdir(x,y) _mkdir(x)
# define lstat(x,y) stat(x,y)
#endif

static int _gettemp( char *path, int *doopen, int domkdir );

int bsd_mkstemp( char *path )
{
  int fd;

  return ( _gettemp( path, &fd, 0 ) ? fd : -1 );
}

char *bsd_mkdtemp( char *path )
{
  return ( _gettemp( path, ( int * ) NULL, 1 ) ? path : ( char * ) NULL );
}

static int _gettemp( char *path, int *doopen, int domkdir )
{
  char *start, *trv;
  struct stat sbuf;
  pid_t pid;

  if ( doopen && domkdir ) {
    errno = EINVAL;
    return ( 0 );
  }

  pid = getpid(  );
  for ( trv = path; *trv; ++trv );      /*%< extra X's get set to 0's */
  while ( *--trv == 'X' ) {
    *trv = ( pid % 10 ) + '0';
    pid /= 10;
  }

  /*
   * check the target directory; if you have six X's and it
   * doesn't exist this runs for a *very* long time.
   */
  if ( doopen || domkdir ) {
    for ( start = trv + 1;; --trv ) {
      if ( trv <= path )
        break;
      if ( *trv == '/' ) {
        *trv = '\0';
        if ( stat( path, &sbuf ) )
          return ( 0 );
        if ( !S_ISDIR( sbuf.st_mode ) ) {
          errno = ENOTDIR;
          return ( 0 );
        }
        *trv = '/';
        break;
      }
    }
  }

  for ( ;; ) {
    if ( doopen ) {
      if ( ( *doopen =
             open( path, O_CREAT | O_EXCL | O_RDWR, 0600 ) ) >= 0 )
        return ( 1 );
      if ( errno != EEXIST )
        return ( 0 );
    } else if ( domkdir ) {
      if ( mkdir( path, 0700 ) == 0 )
        return ( 1 );
      if ( errno != EEXIST )
        return ( 0 );
    } else if ( lstat( path, &sbuf ) )
      return ( errno == ENOENT ? 1 : 0 );

    /* tricky little algorithm for backward compatibility */
    for ( trv = start;; ) {
      if ( !*trv )
        return ( 0 );
      if ( *trv == 'z' )
        *trv++ = 'a';
      else {
        if ( isdigit( *trv ) )
          *trv = 'a';
        else
          ++ * trv;
        break;
      }
    }
  }
 /*NOTREACHED*/}

==> excel2xoonips-2.0.0.tar.gz: excel2xoonips-2.0.0/excel2xoonips/bsd_mktemp.c <==
/*
 * Copyright (c) 1987, 1993
 *    The Regents of the University of California.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 	This product includes software developed by the University of
 * 	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Portions Copyright (c) 1993 by Digital Equipment Corporation.
 * 
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies, and that
 * the name of Digital Equipment Corporation not be used in advertising or
 * publicity pertaining to distribution of the document or software without
 * specific, written prior permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <sys/types.h>
#include <sys/stat.h>

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif

#ifdef WIN32
# define mkdir(x,y) _mkdir(x)
# define lstat(x,y) stat(x,y)
#endif

static int _gettemp( char *path, int *doopen, int domkdir );

int bsd_mkstemp( char *path )
{
  int fd;

  return ( _gettemp( path, &fd, 0 ) ? fd : -1 );
}

char *bsd_mkdtemp( char *path )
{
  return ( _gettemp( path, ( int * ) NULL, 1 ) ? path : ( char * ) NULL );
}

static int _gettemp( char *path, int *doopen, int domkdir )
{
  char *start, *trv;
  struct stat sbuf;
  pid_t pid;

  if ( doopen && domkdir ) {
    errno = EINVAL;
    return ( 0 );
  }

  pid = getpid(  );
  for ( trv = path; *trv; ++trv );      /*%< extra X's get set to 0's */
  while ( *--trv == 'X' ) {
    *trv = ( pid % 10 ) + '0';
    pid /= 10;
  }

  /*
   * check the target directory; if you have six X's and it
   * doesn't exist this runs for a *very* long time.
   */
  if ( doopen || domkdir ) {
    for ( start = trv + 1;; --trv ) {
      if ( trv <= path )
        break;
      if ( *trv == '/' ) {
        *trv = '\0';
        if ( stat( path, &sbuf ) )
          return ( 0 );
        if ( !S_ISDIR( sbuf.st_mode ) ) {
          errno = ENOTDIR;
          return ( 0 );
        }
        *trv = '/';
        break;
      }
    }
  }

  for ( ;; ) {
    if ( doopen ) {
      if ( ( *doopen = open( path, O_CREAT | O_EXCL | O_RDWR, 0600 ) ) >= 0 )
        return ( 1 );
      if ( errno != EEXIST )
        return ( 0 );
    } else if ( domkdir ) {
      if ( mkdir( path, 0700 ) == 0 )
        return ( 1 );
      if ( errno != EEXIST )
        return ( 0 );
    } else if ( lstat( path, &sbuf ) )
      return ( errno == ENOENT ? 1 : 0 );

    /* tricky little algorithm for backward compatibility */
    for ( trv = start;; ) {
      if ( !*trv )
        return ( 0 );
      if ( *trv == 'z' )
        *trv++ = 'a';
      else {
        if ( isdigit( *trv ) )
          *trv = 'a';
        else
          ++ * trv;
        break;
      }
    }
  }
 /*NOTREACHED*/}
